const { PropTypes } = React;
const { Provider, connect } = ReactRedux;
const { createStore, combineReducers } = Redux;
const { reduxForm } = ReduxForm;
const formReducer = ReduxForm.reducer;

const l10nSet = set => {
  return code => {
    const msg = set[code];
    if (msg) {
      return msg;
    }
    return code;
  };
};
const _e = l10nSet(window.regFormErrorMessages);
const _l = l10nSet(window.regFormExtraLabels);

const anyFieldTouched = (fields, ...names) => {
  if (!fields) return false;
  for (name of names) {
    if (fields[name] && fields[name].touched) {
      return true;
    }
  }
  return false;
};

const validate = (values, props) => {
  const errors = {};

  if ("email" in values) {
    if (!values.email) {
      errors.email = _e("required");
    } else if (
      !/^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i.test(values.email)
    ) {
      errors.email = _e("email.invalid");
    }
  }

  if ("phone" in values && values.phone) {
    if (
      !libphonenumber.isValidNumber(
        libphonenumber.parse(values.phone, { country: { default: "CZ" } })
      )
    ) {
      errors.phone = _e("phone.invalid");
    }
  }

  if ("password" in values && "password2" in values) {
    if (!values.password) {
      errors.password = _e("required");
    }
    if (!values.password2) {
      errors.password2 = _e("required");
    } else if (values.password !== values.password2) {
      errors.password2 = _e("password2.no-match");
    }
  }

  const pinfo = personInfo(values);
  if (pinfo === null) {
    errors._error = _e("person-info.required");
  }
  if (values.selectPerson == "NEW") {
    if (!values.firstname) {
      errors.firstname = _e("required");
    }
    if (!values.surname) {
      errors.surname = _e("required");
    }
    if (!values.birth) {
      errors.birth = _e("required");
    }
    if (!values.sex) {
      errors.sex = _e("required");
    }

    if (!values.membership) {
      errors.membership = _e("required");
    }
    if (!values.licenseAccepted) {
      errors.licenseAccepted = _e("required");
    }
  }

  return errors;
};

const addQueryData = (url, data) => {
  const encoded = encodeURIComponent(
    typeof data === "object" ? JSON.stringify(data) : data
  );
  if (url.indexOf("?") !== -1) {
    return url + "&data=" + encoded;
  } else {
    return url + "?data=" + encoded;
  }
};

const asyncValidate = (values, dispatch) => {
  return new Promise((resolve, reject) => {
    if (!values.email && !values.phone) {
      resolve();
    }

    if (values.email) {
      $.getJSON(
        addQueryData(window.regFormCheckEmailLink, values.email),
        function(result) {
          if (!result || !result.email) {
            resolve();
          } else {
            reject(result);
          }
        }
      ).fail(function(jqxhr, textStatus, error) {
        reject("Request Failed: " + (textStatus + ", " + error));
      });
    }

    if (values.phone) {
      $.getJSON(
        addQueryData(window.regFormCheckPhoneLink, values.phone),
        function(result) {
          if (!result || !result.phone) {
            resolve();
          } else {
            reject(result);
          }
        }
      ).fail(function(jqxhr, textStatus, error) {
        reject("Request Failed: " + (textStatus + ", " + error));
      });
    }
  });
};

const RegFormInput = ({ field, control }) => {
  const attrs = {
    type: control.type || "text"
  };
  if (control.disabled) {
    attrs.disabled = true;
  }

  switch (control.type) {
    case "radiogroup":
      const values = options(control.key, control);
      const inline = control.flags.includes("inline");
      const WrapTag = inline ? "span" : "div";
      const labelClass = inline ? "radio inline" : "radio";

      const handleChange = name => {
        return event => {
          optionSelected(control.key, name, event);
        };
      };

      return (
        <WrapTag className={"radiogroup " + control.key}>
          {values.map(([name, value]) => {
            return (
              <label className={labelClass}>
                <input
                  {...field}
                  {...attrs}
                  type="radio"
                  value={name}
                  onClick={handleChange(name)}
                />
                {value}
              </label>
            );
          })}
        </WrapTag>
      );

    case "select":
      const opts = options(control.key, control);
      return (
        <select {...field} {...attrs}>
          {opts.map(([name, value]) => {
            return <option value={name}>{value}</option>;
          })}
        </select>
      );

    default:
      return <input {...field} {...attrs} onKeyUp={field.onBlur} />;
  }
};

let RegForm = (
  { fields, controls, error, dirty, asyncValidate, handleSubmit, submitting }
) => {
  const finalSubmit = controls.state == "MEMBER_SELECT";
  return (
    <form onSubmit={handleSubmit} className="form form-horizontal">
      {Object.keys(fields).map((name, index) => {
        const field = fields[name];
        const control = controls[name];
        if (control.type === "title") {
          return (
            <div>
              {index > 0 && <hr />}
              <h3>{control.label}</h3>
            </div>
          );
        }

        let validationResult;
        if (field.touched) {
          if (field.error) {
            validationResult += " error";
          } else if (field.value) {
            validationResult += " success";
          }
        }
        const classes = "control-group" +
          (validationResult ? " " + validationResult : "");
        return (
          <div className={classes}>
            <label className="control-label">
              {control.required &&
                <span>
                  <strong>{control.label}</strong>
                  <sup><i className="icon-asterisk" aria-hidden="true" /></sup>
                </span>}
              {!control.required && control.label}
            </label>
            <div className="controls">
              <RegFormInput {...{ field, control }} />
              {!control.flags.includes("novisualvalidation") &&
                <span className="help-inline">
                  {field.error &&
                    field.touched &&
                    <span>
                      <i
                        className="icon-remove icon-large"
                        aria-hidden="true"
                      />
                      {" "}
                      {field.error}
                    </span>}
                  {!field.error &&
                    field.touched &&
                    field.value &&
                    <i className="icon-ok icon-large" aria-hidden="true" />}
                </span>}
              {control.flags
                .filter(flag => flag.startsWith("help:"))
                .map(help => {
                  return (
                    <span className="help-block">
                      {help.substring(5)}
                    </span>
                  );
                })}
            </div>
          </div>
        );
      })}
      {error && dirty && <div className="alert alert-error">{error}</div>}
      <div className="control-group">
        <div className="controls">
          <button
            className="btn btn-primary"
            onClick={handleSubmit}
            disabled={submitting}
          >
            {submitting && <i className="icon-spinner icon-spin icon-large" />}
            {" "}
            {finalSubmit ? controls.submit : controls.subsubmit}
          </button>
        </div>
      </div>
    </form>
  );
};
RegForm.propTypes = {
  fields: PropTypes.array,
  controls: PropTypes.object
};
RegForm = reduxForm(
  {
    form: "data",
    validate,
    asyncValidate,
    asyncBlurFields: ["email", "phone"],
	initialValues: { "email": window.regFormGivenEmail },
    onSubmit: (formData, dispatch) => {
      return new Promise(function(resolve, reject) {
        dispatch({ type: "UPDATE_DATA", payload: formData });
        Promise.resolve(nextAction(formData))
          .then(action => {
            if (Array.isArray(action)) {
              for (let a of action) {
                dispatch(a);
              }
            } else {
              dispatch(action);
            }

            const storeState = store.getState();
            if (storeState.control.state === "DONE") {
              $.getJSON(
                addQueryData(
                  window.regFormSubmitDataLink,
                  registrationInfo(storeState.data)
                ),
                function(result) {
                  if (!result.error) {
                    dispatch({
                      type: "REGISTRATION_PASSED",
                      message: result.message
                    });
                  } else {
                    dispatch({
                      type: "REGISTRATION_FAILED",
                      message: result.message
                    });
                  }
                  resolve();
                }
              ).fail(function(jqxhr, textStatus, error) {
                reject("Request Failed: " + (textStatus + ", " + error));
              });
            } else {
              resolve();
            }
          })
          .catch(reason => {
            console.log("Action failed:", reason);
            dispatch({ type: "REGISTRATION_FAILED", message: reason });
          });
      });
    }
  },
  state => {
    return {
      fields: Object.keys(state.control)
        .filter(key => state.control && state.control[key] && state.control[key].visible)
        .sort((a, b) => state.control[a].order - state.control[b].order),
      controls: state.control
    };
  }
)(RegForm);

const options = (name, control) => {
  switch (name) {
    case "sex":
      return [["FEMALE", _l("female")], ["MALE", _l("male")]];
    case "selectPerson":
      return [
        ...control.valueOptions.map(([key, value]) => [
          key,
          _l("existing-membership.prefix") + " " + value
        ]),
        ["NEW", _l("new-membership")],
        ["NONE", _l("no-membership")]
      ];
    case "membership":
      return control.valueOptions;
    default:
      return [];
  }
};

const optionSelected = (name, value, event) => {
  switch (name) {
    case "selectPerson":
      switch (value) {
        case "NEW":
          store.dispatch({ type: "SHOW_REGISTER_NEW" });
          break;
        default:
          store.dispatch({ type: "HIDE_REGISTER_NEW" });
          break;
      }
      break;
  }
};

const personInfo = data => {
  if (
    !("firstname" in data ||
      "surname" in data ||
      "sex" in data ||
      "birth" in data)
  )
    return;
  if (!data.firstname && !data.surname && !data.sex && !data.birth) return null;
  return [data.firstname, data.surname, data.birth, data.sex];
};

const registrationInfo = data => {
  return [
    data.email,
    data.phone,
    data.password,
    data.password2,
    ...personInfo(data),
    data.selectPerson,
    data.membership,
    data.licenseAccepted
  ];
};

const nextAction = data => {
  const pinfo = personInfo(data);
  if (pinfo) {
    const actionType = "selectPerson" in data && !data.selectPerson
      ? "RETRY"
      : "NEXT";
    return Promise.all([
      new Promise(function(resolve, reject) {
        $.getJSON(window.regFormGetClubsLink, function(result) {
          resolve({ type: "UPDATE_CLUBS", clubs: result });
        }).fail(function(jqxhr, textStatus, error) {
          reject("Request Failed: " + (textStatus + ", " + error));
        });
      }),
      new Promise(function(resolve, reject) {
        $.getJSON(
          addQueryData(window.regFormGetPersonsLink, pinfo),
          function(result) {
            resolve({ type: actionType, persons: result });
          }
        ).fail(function(jqxhr, textStatus, error) {
          reject("Request Failed: " + (textStatus + ", " + error));
        });
      })
    ]);
  }

  return { type: "NEXT" };
};

const control = (state = window.regFormControls, action) => {
  const newState = Object.assign({}, state);
  switch (action.type) {
    case "NEXT":
      if (action.persons) {
        newState.selectPerson.valueOptions = action.persons;
      }

      switch (state.state) {
        case "USER":
          newState.email.disabled = true;
          newState.phone.disabled = true;
          newState.password.disabled = true;
          newState.password2.disabled = true;
          newState.g3.visible = true;
          newState.selectPerson.visible = true;
          newState.state = "MEMBER_SELECT";
          break;

        case "MEMBER_SELECT":
          newState.state = "DONE";
          break;
      }

      return newState;

    case "RETRY":
      if (action.persons) {
        newState.selectPerson.valueOptions = action.persons;
      }
      return newState;

    case "UPDATE_CLUBS":
      if (action.clubs) {
        newState.membership.valueOptions = action.clubs;
      }
      return newState;

    case "SHOW_REGISTER_NEW":
      newState.membership.visible = true;
      newState.licenseAccepted.visible = true;
      return newState;
    case "HIDE_REGISTER_NEW":
      newState.membership.visible = false;
      newState.licenseAccepted.visible = false;
      return newState;

    case "REGISTRATION_PASSED":
      newState.state = "REGISTRATION_PASSED";
      newState.message = action.message;
      return newState;
    case "REGISTRATION_FAILED":
      newState.state = "REGISTRATION_FAILED";
      newState.message = action.message;
      return newState;

    default:
      return state;
  }
};

const data = (state = {}, action) => {
  switch (action.type) {
    case "UPDATE_DATA":
      return Object.assign({}, state, action.payload);
    default:
      return state;
  }
};

const appReducer = combineReducers({ form: formReducer, control, data });
const store = createStore(appReducer);

let RegApp = ({ step, message, dispatch }) => {
  console.log("RegApp: ", step, message);
  if (step == "REGISTRATION_PASSED") {
    return <div className="alert alert-success">{message}</div>;
  } else if (step == "REGISTRATION_FAILED") {
    return <div className="alert alert-error">{message}</div>;
  } else {
    return <RegForm />;
  }
};
RegApp.propTypes = {
  step: PropTypes.string,
  message: PropTypes.string
};
RegApp = connect(state => {
  return {
    step: state.control.state,
    message: state.control.message
  };
})(RegApp);

ReactDOM.render(
  <Provider store={store}>
    <RegApp />
  </Provider>,
  document.getElementById("regForm")
);
